<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 15/10/2015
 * Time: 14:57
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Factory\Release as ReleaseFactory;
use Factory\Package as PackageFactory;
use Factory\Product as ProductFactory;
use Factory\ProductVersion as ProductVersionFactory;
use Factory\ProductVersionPackages as ProductVersionPackagesFactory;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Devops\Entities\ProductVersion;
use Factory\ZhUserAllocation as ZhUserAllocationFactory;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\Release as ReleaseEntity;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\Package as PackageEntity;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\PackageContents as PackageContentsEntity;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\DevOps\Util\SocketClient;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories as PmRepositories;
use Zeedhi\DevOps\Model\Projectmanagement\Entities as PmEntities;
use Zeedhi\DevOps\Model\Devops\Entities as DevOpsEntities;
use Zeedhi\DevOps\Model\Profile\Entities as ProfileEntities;
use Zeedhi\DevOps\Model\Configurationmanager\Entities as ConfigurationEntities;
use Zeedhi\DevOps\Model\Devops\Repositories as DevOpsRepositories;
use Zeedhi\DevOps\Model\Profile\Repositories as ProfileRepositories;
use Zeedhi\DevOps\Model\Configurationmanager\Repositories as ConfigurationRepositories;
use Util\Messages;

class Release
{
    const GENERATED_TASK_NAME = 'Generated task for resolve conflict in the release ';
    const CONFLICT_SUFIX_TASK_TAG_NAME = '[CDO]';

    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var ReleaseFactory $releaseFactory */
    protected $releaseFactory;
    /** @var PackageFactory $packageFactory */
    protected $packageFactory;
    /** @var ProductFactory $productFactory */
    protected $productFactory;
    /** @var ProductVersionFactory $productVersionFactory */
    protected $productVersionFactory;
    /** @var ProductVersionPackagesFactory $productVersionPackagesFactory */
    protected $productVersionPackagesFactory;
    /** @var ZhUserAllocationFactory $zhUserAllocationFactory */
    protected $zhUserAllocationFactory;
    /** @var PmRepositories\TaskStatus $taskStatusRepository */
    protected $taskStatusRepository;
    /** @var PmRepositories\ProjectRelationship $projectRelationshipRepository */
    protected $projectRelationshipRepository;
    /** @var ConfigurationRepositories\Subrelease $subreleaseRepository */
    protected $subreleaseRepository;
    /** @var ConfigurationRepositories\Release $releaseRepository */
    protected $releaseRepository;
    /** @var ProfileRepositories\ZhUser $userRepository */
    protected $userRepository;
    /** @var DevOpsRepositories\ZhUserAllocation $userAllocationRepository */
    protected $userAllocationRepository;
    /** @var PmRepositories\Allocation $allocationRepository */
    protected $allocationRepository;
    /** @var PmRepositories\Project $allocationRepository */
    protected $projectRepository;
    /** @var PmRepositories\Classification $classificationRepository */
    protected $classificationRepository;
    /** @var PmRepositories\Resource $resourceRepository */
    protected $resourceRepository;
    /**@var Subrelease*/
    private $subreleaseService;
    /**
     * @var Environment
     */
    private $environment;

    /**
     * @param EntityManager $entityManager
     * @param Environment $environment
     * @param ReleaseFactory $releaseFactory
     * @param PackageFactory $packageFactory
     * @param ProductFactory $productFactory
     * @param ProductVersionFactory $productVersionFactory
     * @param ProductVersionPackagesFactory $productVersionPackagesFactory
     * @param ZhUserAllocationFactory $zhUserAllocationFactory
     * @param Subrelease $subreleaseService
     */
    public function __construct(EntityManager $entityManager, Environment $environment,ReleaseFactory $releaseFactory, PackageFactory $packageFactory,
                                ProductFactory $productFactory, ProductVersionFactory $productVersionFactory,
                                ProductVersionPackagesFactory $productVersionPackagesFactory,
                                ZhUserAllocationFactory $zhUserAllocationFactory, Subrelease $subreleaseService)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->releaseFactory = $releaseFactory;
        $this->packageFactory = $packageFactory;
        $this->productFactory = $productFactory;
        $this->productVersionFactory = $productVersionFactory;
        $this->productVersionPackagesFactory = $productVersionPackagesFactory;
        $this->zhUserAllocationFactory = $zhUserAllocationFactory;
        $this->subreleaseService = $subreleaseService;

        //DevOps Repositories
        $this->subreleaseRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease');
        $this->releaseRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release');
        $this->userRepository = $this->entityManager->getRepository(Utilities::PROFILE_ENTITY_DIR . 'ZhUser');
        $this->userAllocationRepository = $this->entityManager->getRepository(Utilities::DEVOPS_ENTITY_DIR . 'ZhUserAllocation');
        //PM Repositories
        $this->repositoryRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Repository');
        $this->taskStatusRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'TaskStatus');
        $this->projectRelationshipRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'ProjectRelationship');
        $this->projectRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Project');
        $this->classificationRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Classification');
        $this->resourceRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Resource');
    }

    /**
     * @param $id
     * @param $name
     * @param $productId
     * @param $userId
     * @param $productServerId
     * @param $updateVersion
     * @param $fixRelease
     * @param $status
     * @param $version
     * @param $revision
     * @param $build
     * @param $description
     * @param $isActive
     * @throws \Exception
     * @return int
     */
    public function saveRelease($id, $name, $productId, $userId, $productServerId, $updateVersion, $fixRelease,
                                $status, $version, $revision, $build, $description, $isActive)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $productServer = $this->entityManager->find(Utilities::SERVER_MANAGER_ENTITY_DIR . 'ProductServer', $productServerId);
            $product = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Product', $productId);

            if ($id) {
                /** @var ReleaseEntity $release */
                $release = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release', $id);
            } else {
                $release = new ReleaseEntity();
            }

            $release = $this->releaseFactory->setReleaseRecords($release, $name, $product, $productServer,
                $updateVersion, $fixRelease, $status, $version, $revision, $build, $description, $isActive);

            $this->entityManager->persist($release);
            $this->entityManager->flush();

            if (!$id) {
                $productPackages = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Package')
                    ->findBy(array('product' => $product));

                /** @var PackageEntity $prodPackage */
                foreach ($productPackages as $prodPackage) {
                    /** @var PackageEntity $package */
                    $package = new PackageEntity();
                    $package = $this->packageFactory->setPackageRecords($package, $release, $prodPackage->getName(),
                        $prodPackage->getPath(), $prodPackage->getDescription(), $prodPackage->getFileName(), $prodPackage->getPathToDeploy());
                    $this->entityManager->persist($package);
                    $this->entityManager->flush();

                    $packagesContents = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'PackageContents')
                        ->findBy(array('package' => $prodPackage));

                    /** @var PackageContentsEntity $packContents */
                    foreach ($packagesContents as $packContents) {
                        /** @var PackageContentsEntity $packageContents */
                        $packageContents = new PackageContentsEntity();
                        $packageContents = $this->packageFactory->setPackageContentsRecords($packageContents, $package, $packContents->getName(),
                            $packContents->getPath(), $packContents->getFileName());
                        $this->entityManager->persist($packageContents);
                    }
                }
            }

            $this->savereleaseUsers($release, $userId);
            if ($release->getIsActive() == Utilities::IS_ACTIVE_NO) {
                $this->deactivateSubreleases($release);
            }

            $this->entityManager->flush();
            $connection->commit();

            return $release->getId();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    public function removePathReleaseForController($releaseId){
        /** @var ConfigurationEntities\Release $release */
        $release = $this->releaseRepository->find($releaseId);
        $this->removePathRelease($release);
    }


    public function removePathRelease(ConfigurationEntities\Release $release)
    {
        try{
            /** @var ConfigurationEntities\SubRelease[] $subReleases */
            $subReleases = $this->subreleaseRepository->findBy(array('release' => $release));
            foreach ($subReleases as $subRelease) {
                $this->subreleaseService->removePathSubRelease($subRelease);
            }
            if ($release->getProductServer()) {
                $pathRelease = $this->getReleasePath($release);
                $wsEmail = $release->getProductServer()->getServer()->getWsemail();
                if ($wsEmail && $pathRelease) {
                    $socket = SocketClient::factory();
                    $socket->removeDir($wsEmail, $pathRelease);
                }
            }

        }catch (\Exception $e){ }
    }

    public function getReleasePath(ConfigurationEntities\Release $release){
        $masterRepository = $this->repositoryRepository->findOneBy(array('product' => $release->getProduct(),
            'isMaster' => PmEntities\Repository::IS_MASTER));
        $baseDir = '';
        if($release->getProductServer()->getPath()){
            $baseDir = $release->getProductServer()->getPath() . DIRECTORY_SEPARATOR;
        }
        return $baseDir . $masterRepository->getPath() . '_PROD_' . $release->getProduct()->getId() . '_REL_' . $release->getId();
    }


    /**
     * @param $packageId
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Exception
     */
    public function removePackage($packageId)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $package = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Package', $packageId);

            $packagesContents = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'PackageContents')
                ->findBy(array('package' => $package));

            /** @var PackageContentsEntity $packContents */
            foreach ($packagesContents as $packContents) {
                $this->entityManager->remove($packContents);
            }
            $this->entityManager->flush();

            $this->entityManager->remove($package);
            $this->entityManager->flush();

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $releaseId
     * @return bool
     * @throws \Exception
     */
    public function checkPermission($releaseId) {
        $currentUserId     = $this->environment->getCurrentUserId();
        $currentCdOperador = $this->environment->getCurrentCdoperador();
        $resource = InstanceProvider::getRepository()->getResource()->findOneBy(array('externalCode' => $currentCdOperador));
        if($resource) {
            $zhUser = InstanceProvider::getRepository()->getZhUser()->findOneBy(array('externalResId' => $resource->getId()));
            if($zhUser) {
                $currentUserId = $zhUser->getId();
                $this->environment->setCurrentUserId($currentUserId);
            }
        }
        $user = $this->userAllocationRepository->findOneBy(array('user' => $currentUserId, 'release' => $releaseId));
        if (!$user && !empty($currentUserId)) {
            throw new \Exception(Messages::NO_PERMISSION_TO_FINISH_RELEASE);
        }
        return true;
    }


    /**
     * @param $releaseId
     * @return array
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     * @throws \Doctrine\ORM\TransactionRequiredException
     */
    public function findPackagesByRelease($releaseId)
    {
        $packagesWithContents = array();

        /** @var ReleaseEntity $release */
        $release = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release', $releaseId);


        $packages = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Package')
            ->findBy(array('release' => $release));

        /** @var PackageEntity $package */
        foreach ($packages as $package) {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\PackageContents $packagesContRep */
            $packagesContRep = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'PackageContents');

            $packagesWithContents[] = array(
                'ID' => $package->getId(),
                'NAME' => $package->getName(),
                'PATH' => $package->getPath(),
                'PATH_UNCHANGED' => $package->getPath(),
                'PATH_TO_DEPLOY' => $package->getPathToDeploy(),
                'FILE_NAME' => $package->getFileName(),
                'FILE_NAME_NOT_PATH' => $package->getFileName(),
                'DESCRIPTION' => $package->getDescription(),
                'PACKAGE_CONTENTS' => $packagesContRep->findByPackage($package->getId())
            );
        }

        return $packagesWithContents;
    }

    /**
     * @param $releaseId
     * @param $version
     * @param $revision
     * @param $build
     * @param $releaseStatus
     * @param $subReleaseStatus
     * @param $isProjectAssociation
     * @param $packages
     * @param $modulesConfiguration
     * @throws \Exception
     */
    public function homologateRelease($releaseId, $version, $revision, $build, $releaseStatus, $subReleaseStatus,
                                      $isProjectAssociation, $packages, $modulesConfiguration = '')
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $release = $this->setReleaseRecordsFinish($releaseId, $version, $revision, $build, $releaseStatus);
            $product = $release->getProduct();
            $this->setSubReleasesRecordsFinish($release, $version, $revision, $build, $subReleaseStatus);
            $this->setProductRecordsFinish($version, $revision, $build, $release);
            $productVersion = $this->setProductVersionRecordsFinish($version, $revision, $build, $product);
            $this->setProductVersionPackagesRecordsFinish($productVersion, $packages, $release->getProductServer());
            if ($isProjectAssociation) {
                $this->concludeProjects($release);
            }
            if(!empty($modulesConfiguration)) {
                $this->subreleaseService->setModulesConfiguration($product, $modulesConfiguration, $productVersion, true);
            }

            $this->entityManager->flush();

            $numberLts = $product->getNumberLtsAvailable();
            $numberLts = !is_null($numberLts) && $numberLts == 0 ? 1 : $numberLts;
            if(!empty($numberLts)) {
                $this->removeLtsFromOldVersions($product, $numberLts);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $release
     * @param $version
     * @param $revision
     * @param $build
     * @param $status
     * @return \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease
     */
    private function setSubReleasesRecordsFinish($release, $version, $revision, $build, $status)
    {
        $subReleases = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease')
            ->findBy(array('release' => $release));

        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
        foreach ($subReleases as $subrelease) {
            $subrelease->setStatus($status);
            $subrelease->setVersion($version);
            $subrelease->setRevision($revision);
            $subrelease->setBuild($build);
            $this->entityManager->persist($subrelease);
        }
    }

    /**
     * @param $releaseId
     * @param $version
     * @param $revision
     * @param $build
     * @param $status
     * @return \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release
     */
    private function setReleaseRecordsFinish($releaseId, $version, $revision, $build, $status)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
        $release = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release', $releaseId);
        $release = $this->releaseFactory->setReleaseRecords($release, $release->getName(), $release->getProduct(),
            $release->getProductServer(), $release->getUpdateVersion(), $release->getIsFixRelease(), $status, $version, $revision, $build, $release->getDescription()
        );
        $this->entityManager->persist($release);
        return $release;
    }

    /**
     * @param $version
     * @param $revision
     * @param $build
     * @param $release
     */
    private function setProductRecordsFinish($version, $revision, $build, $release)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
        $product = $release->getProduct();
        $product = $this->productFactory->setProductRecords($product, $product->getProject(), $product->getName(),
            $product->getBaseUrl(), $product->getDescription(), $version, $revision, $build, $product->getIsActive(), $product->getNumberLtsAvailable()
        );
        $this->entityManager->persist($product);
    }

    /**
     * @param $version
     * @param $revision
     * @param $build
     * @param $product
     * @return \Zeedhi\DevOps\Model\Devops\Entities\ProductVersion
     */
    private function setProductVersionRecordsFinish($version, $revision, $build, $product)
    {
        $productVersion = new ProductVersion();
        $this->productVersionFactory->setProductVersionRecords($productVersion, $product, $version, $revision, $build,
            ProductVersion::IS_REVISION_YES, ProductVersion::IS_BUILD_NO, ProductVersion::IS_LTS_YES, ProductVersion::IS_BETA_YES);
        $this->entityManager->persist($productVersion);
        return $productVersion;
    }
    
    
    public function inactiveReleasesFix()
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $projectStatus = PmEntities\Project::PROJECT_CANCELED_STATUS;
            $releaseRegisters = $this->releaseRepository->inactiveReleasesFix($projectStatus);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
       
        return $projectStatus;
    }

    /**
     * @param $productVersion
     * @param $packages
     * @param $productServer
     */
    private function setProductVersionPackagesRecordsFinish($productVersion, $packages, $productServer)
    {
        $this->subreleaseService->setProductVersionPackagesRecordsFinish($productVersion, $packages, $productServer);
    }

    public function concludeProjectParent(PmEntities\Project $project, PmEntities\TaskStatus $projectStatusConcluded)
    {
        $this->subreleaseService->concludeProjectParent($project, $projectStatusConcluded);
    }

    /**
     * @param $release
     */
    private function concludeProjects($release)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
        $project = $release->getProject();
        /** @var PmEntities\TaskStatus $projectStatusConcluded */
        $projectStatusConcluded = $this->taskStatusRepository->findOneBy(array('isProject' => 'Y', 'control' => Utilities::PROJECT_STATUS_CONTROL_CONCLUDED));
        $project->setProjectStatus($projectStatusConcluded);
        /** @var ConfigurationEntities\Subrelease[] $subReleases */
        $subReleases = $this->subreleaseRepository->findBy(array('release' => $release));
        foreach($subReleases as $subRelease){
            $subReleaseProject = $subRelease->getProject();
            $subReleaseProject->setProjectStatus($projectStatusConcluded);
            $this->entityManager->persist($subReleaseProject);
            $this->entityManager->flush();
            $this->concludeProjectParent($subReleaseProject, $projectStatusConcluded);

           
            $projectRelationship = $this->projectRelationshipRepository->findBy(array('project' => $subReleaseProject, 'type' => PmEntities\ProjectRelationship::TYPE_TEST));
        
           /** @var PmEntities\ProjectRelationship $relationship */
            foreach($projectRelationship as $relationship) {
                $testProject = $relationship->getProjectRelationship();
                $testProject->setProjectStatus($projectStatusConcluded);
                $this->entityManager->persist($testProject);
                $this->entityManager->flush();
                $this->concludeProjectParent($testProject, $projectStatusConcluded);
            }
        }
        $this->entityManager->persist($project);
        $this->entityManager->flush();
        $this->concludeProjectParent($project, $projectStatusConcluded);
    }

    /**
     * @param $productId
     * @return array
     */
    public function findPackageServersByProduct($productId, $filter = array())
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $organizationId = $this->environment->getCurrentOrganizationId();
        return $this->releaseRepository->findPackageServersByProduct($productId, $organizationId, $filterString);
    }

    public function validateReleaseServerMnemonic($releaseId, $typeEvent){
        /** @var ConfigurationEntities\Release $release */
        $release = $this->releaseRepository->find($releaseId);
        return $this->subreleaseService->productFilesService->validateReleaseServerMnemonic($release, $typeEvent);
    }

    /**
     * @param $subreleaseId
     * @return array
     */
    public function findIssuesPartialCompletedBySubreleaseId($subreleaseId)
    {
        $organizationId = $this->environment->getCurrentOrganizationId();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Task $taskRepository */
        $taskRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Task');
        return $taskRepository->getIssuesPartialCompletedBySubrelease($subreleaseId, $organizationId);
    }

    /**
     * @param $releaseId
     * @throws \Exception
     */
    public function removeRelease($releaseId)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
            $release = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release', $releaseId);
            $subreleases = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease')
                ->findBy(array('release' => $release));
            $this->removeSubreleasesAndBranches($subreleases);
            $release->setIsActive(Utilities::IS_ACTIVE_DELETED);
            $this->entityManager->persist($release);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $subreleases
     */
    private function removeSubreleasesAndBranches($subreleases)
    {
        foreach ($subreleases as $subrelease) {
            $branches = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Branch')
                ->findBy(array('subrelease' => $subrelease));

            foreach ($branches as $branch) {
                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Branch $branch */
                $branch->setSubrelease(null);
                $this->entityManager->persist($branch);
            }
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
            $subrelease->setIsActive(Utilities::IS_ACTIVE_DELETED);
            $this->entityManager->persist($subrelease);
        }
    }

    /**
     * @param $release
     */
    private function deactivateSubreleases($release)
    {
        $subreleases = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease')
            ->findBy(array('release' => $release));
        foreach ($subreleases as $subrelease) {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
            $subrelease->setIsActive(Utilities::IS_ACTIVE_NO);
            $this->entityManager->persist($subrelease);
        }
    }

    /**
     * @param $conditions
     * @return array
     */
    public function findRelease($conditions, $page = null, $pageSize = null) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filters = Utilities::createQueryFiltered($conditions);
        $filters = $filters.' AND ORGANIZATION_ID = '.$this->environment->getCurrentOrganizationId().' ';
        $releaseRegisters = $this->releaseRepository->getRelease($filters, $limit, $offset);

        /** @var ConfigurationEntities\Release $release */
        foreach($releaseRegisters as &$release) {
            if($release['USER_ID']) {
                $release['USER_ID'] = Utilities::setSelectMultiInQuery($release['USER_ID'], true);
            }
            if($release['ZH_USER']){
                $release['ZH_USER'] = Utilities::setSelectMultiInQuery($release['ZH_USER'], true);
            }
            $release['SUB_PROJECTS'] = $this->getSubreleaseWithProject($release['ID']);
        }

        return $releaseRegisters;
    }

    private function savereleaseUsers(ConfigurationEntities\Release $release, $userIds) {
        foreach ($userIds as $userId) {
            /** @var ProfileEntities\ZhUser $user */
            $user = $this->userRepository->find($userId);
            /** @var DevOpsEntities\ZhUserAllocation $userAllocation */
            $userAllocation = $this->userAllocationRepository->findOneBy(array('release' => $release, 'user' => $user));
            if (empty($userAllocation)) {
                $userAllocation = $this->zhUserAllocationFactory->setZhUserAllocationRecords($userAllocation, $user, null, $release, null);
                $this->entityManager->persist($userAllocation);
            }
        }

        $userAllocations = $this->userAllocationRepository->findBy(array('release' => $release));
        foreach ($userAllocations as $zhUserAllocation) {
            /** @var DevOpsEntities\ZhUserAllocation $zhUserAllocation */
            if (!in_array($zhUserAllocation->getUser()->getId(), $userIds)) {
                $this->entityManager->remove($zhUserAllocation);
            }
        }
    }

    /**
     * @param $resourceId
     * @param $releaseId
     * @return array
     */
    public function getTaskInfoForGenerate($resourceId, $releaseId) {
        $taskInfo = array(
            'CLASSIFICATION' => null,
            'TASK_NAME'      => self::GENERATED_TASK_NAME . $releaseId . ' -- ' . self::CONFLICT_SUFIX_TASK_TAG_NAME,
            'TASK_STATUS'    => Utilities::getTaskStatusNotInitialized()->getId(),
            'STATUS'         => PmEntities\Task::ACTIVE,
            'TIME_PLANNED'   => '8H',
            'TOTAL_TIME'     => '8H'
        );

        /** @var PmEntities\Classification $classification */
        $classification = $this->classificationRepository->findOneBy(array('control' => PmEntities\Classification::CONTROL_CONFLICT_DEVOPS,
            'organization' => $this->environment->getCurrentOrganizationId()));
        if(is_object($classification)) {
            $taskInfo['CLASSIFICATION'] = $classification->getId();
        }

        /** @var PmEntities\Resource $resource */
        $resource = $this->resourceRepository->find($resourceId);
        if(is_object($resource)) {
            $rating      = $resource->getRating() ? $resource->getRating() : 1;
            $timePlanned = Utilities::timeToSeconds($taskInfo['TIME_PLANNED']);
            $duration    = $timePlanned / $rating;
            $taskInfo['TOTAL_TIME'] = Utilities::secondsToTime($duration);
        }

        return $taskInfo;
    }

    private function getSubreleaseWithProject($releaseId) {
        $result = array();
        $subReleases = $this->subreleaseRepository->findBy(array('release' => $releaseId));
        /** @var ConfigurationEntities\Subrelease $subRelease */
        foreach($subReleases as $subRelease) {
            $result[$subRelease->getId()] = null;
            if($subRelease->getProject()) {
                $result[$subRelease->getId()] = array(
                    'PROJECT_ID'   => $subRelease->getProject()->getId(),
                    'PROJECT_NAME' => $subRelease->getProject()->getName()
                );
            }
        }
        return $result;
    }

    private function removeLtsFromOldVersions(ConfigurationEntities\Product $product, $numberLts) {
        $productVersionRepository = InstanceProvider::getRepository()->getProductVersion();
        $versions = $productVersionRepository->findLtsVersionWithQuantityFilter (
            $product->getId(), $numberLts, InstanceProvider::getLibEnvironment()->getCurrentOrganizationId()
        );
        foreach($versions as $version) {
            /** @var ProductVersion $productVersion */
            $productVersion = $productVersionRepository->find($version['ID']);
            $productVersion->setIsLts(ProductVersion::IS_LTS_NO);
            $this->entityManager->persist($productVersion);
        }
    }

    public function findResponsesOfIssuesAndTasksByRelease($releaseId, $conditions)
    {
        $filter = Utilities::createQueryFiltered($conditions);
        $organizationId = $this->environment->getCurrentOrganizationId();
        $subReleases = $this->subreleaseRepository->findSubreleaseByReleaseId($releaseId);
        $subReleaseIds = array_column($subReleases, 'ID');
        return InstanceProvider::getRepository()->getTask()->findResponsesOfIssuesAndTasks($subReleaseIds, $organizationId, $filter);
    }

    /**
     * @param $releaseId
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function setPackageCreatedStatusInSubRelease($releaseId)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
        $release = $this->releaseRepository->find($releaseId);
        $release->setStatus(ConfigurationEntities\Release::PACKAGE_CREATED_STATUS);
        $this->entityManager->persist($release);
        $this->entityManager->flush();
    }

    public function setStatusInRelease($releaseId, $status)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
        $release = $this->releaseRepository->find($releaseId);
        $release->setStatus($status);
        $this->entityManager->persist($release);
        $this->entityManager->flush();
    }
}